@@ -63,7 +63,10 @@ gar_shiny_getAuthUrl <-
            state = getOption("googleAuthR.securitycode"),
            client.id     = getOption("googleAuthR.webapp.client_id"),
            client.secret = getOption("googleAuthR.webapp.client_secret"),
-           scope         = getOption("googleAuthR.scopes.selected")) {
+           scope         = getOption("googleAuthR.scopes.selected"),
+           access_type   = c("online","offline")) {
+    
+    access_type <- match.arg(access_type)
 
     scopeEnc <- paste(scope, sep='', collapse=' ')
     
@@ -75,7 +78,7 @@ gar_shiny_getAuthUrl <-
                    redirect_uri = redirect.uri,
                    scope = scopeEnc,
                    state = state,
-                   access_type = "online",
+                   access_type = access_type,
                    approval_prompt = "auto"))
     message("Auth Token URL: ", url)
     url
@@ -89,7 +92,6 @@ gar_shiny_getAuthUrl <-
 #' @param session The shiny session object.
 #' 
 #' @return The URL of the Shiny App its called from.
-#' @keywords internal
 #' @family shiny auth functions
 gar_shiny_getUrl <- function(session){
   
@@ -108,7 +110,7 @@ gar_shiny_getUrl <- function(session){
     url <- paste0(session$clientData$url_protocol,
                   "//",
                   hostname,
-                  ifelse(hostname == "127.0.0.1",
+                  ifelse(hostname %in% c("127.0.0.1","localhost"),
                          ":",
                          pathname),
                   session$clientData$url_port)
@@ -157,7 +159,7 @@ gar_shiny_getToken <- function(code,
   # content of req will contain access_token, token_type, expires_in
   token <- httr::content(req, type = "application/json")
   
-  # Create a Token2.0 object consistent with the token obtained from scr_auth()
+  # Create a Token2.0 object consistent with the token obtained from gar_auth()
   token_formatted <-
     httr::Token2.0$new(app = gar_app,
                        endpoint = httr::oauth_endpoints("google"),
@@ -251,7 +253,6 @@ reactiveAccessToken <- function(session){
   shiny::reactive({
     ## gets all the parameters in the URL. The auth code should be one of them.
     
-    # if(length(pars$code) > 0) {
     if(!is.null(authReturnCode(session))){
       ## extract the authorization token
       app_url <- gar_shiny_getUrl(session)    
@@ -350,7 +351,9 @@ loginOutput <- function(output_name){
 #' Render a Google API Authentication Login/logout button
 #' 
 #' Use within a Shiny server.R to assign to an output for ui.R.
-#' The login button carries an ActionLink with value "signed_in".
+#' The login button carries an ActionLink with value "signed_in" 
+#'   but as Shiny reloads on pushing it can't be used for detection of login state.  
+#'   Use \code{!is.null(access_token())} instead.
 #' 
 #' @param session A Shiny session object
 #' @param access_token A token generated by \code{reactiveAccessToken}
@@ -358,9 +361,13 @@ loginOutput <- function(output_name){
 #' @param logout_text What the logout text will read on the button
 #' @param login_class The Bootstrap class for the login link
 #' @param logout_class The Bootstrap class for the logout link
+#' @param access_type Online or offline access for the authentication URL. 
+#' @param revoke If TRUE a user on logout will need to re-authenticate.
 #' 
 #' @return An object to assign to output e.g. output$login
 #' 
+#' @seealso \link{revokeEventObserver}
+#' 
 #' @export
 #' @family shiny auth functions
 #' @examples
@@ -397,7 +404,11 @@ loginOutput <- function(output_name){
 #'   access_token  <- reactiveAccessToken(session)
 #' 
 #'   ## Make a loginButton to display using loginOutput
-#'   output$loginButton <- renderLogin(session, access_token())
+#'   ## revoke=TRUE means upon logout a user will need to reauthenticate
+#'   output$loginButton <- renderLogin(session, access_token(), revoke=TRUE)
+#'   
+#'   ## Needed if revoke=TRUE above
+#'   revokeEventObserver(access_token())
 #'
 #'   short_url_output <- eventReactive(input$submit, {
 #'   ## wrap existing function with_shiny
@@ -429,27 +440,143 @@ loginOutput <- function(output_name){
 #'     ))
 #' }
 renderLogin <- function(session, 
-                        access_token, 
+                        access_token,
                         login_text="Login via Google",
                         logout_text="Logout",
                         login_class="btn btn-primary",
-                        logout_class="btn btn-default"){
+                        logout_class="btn btn-default",
+                        access_type = c("online","offline"),
+                        revoke = FALSE){
+  access_type <- match.arg(access_type)
   shiny::renderUI({
     if(is.null(shiny::isolate(access_token))) {
       shiny::actionLink("signed_in",
                  shiny::a(login_text, 
-                          href = gar_shiny_getAuthUrl(gar_shiny_getUrl(session)), 
+                          href = gar_shiny_getAuthUrl(gar_shiny_getUrl(session), 
+                                                      access_type = access_type), 
                    class=login_class, 
                    role="button"))
     } else {
-      shiny::a(logout_text, 
-        href = gar_shiny_getUrl(session), 
-        class=logout_class, 
-        role="button")
+      if(revoke){
+#         logout_button <- shiny::actionLink("revoke",
+#                                            shiny::a(logout_text, 
+#                                                     href = '#', 
+#                                                     class=logout_class, 
+#                                                     role="button")           )
+        
+        logout_button <- shiny::actionButton("revoke", "Revoke Access", 
+                                             href = gar_shiny_getUrl(session), 
+                                             class=logout_class,
+                                             role="button")
+
+      } else {
+        logout_button <- shiny::a(logout_text, 
+                                  href = gar_shiny_getUrl(session), 
+                                  class=logout_class, 
+                                  role="button")
+      }
+      
+      logout_button
+      
     }
   })
 }
 
+#' Listens for a user revoking authentication
+#' 
+#' @description If the parameter \code{revoke} is set to TRUE for \link{renderLogin}
+#'   then this observer is also required in the Shiny server to do the revoking.
+#' 
+#' @param access_token A token generated by \code{reactiveAccessToken}.
+#' @param input the input object from a shinyServer function.
+#' 
+#' @seealso \link{renderLogin}
+#' 
+#' @export
+#' @family shiny auth functions
+#' @examples
+#' \dontrun{
+#' ## in global.R
+#' 
+#' ## create the API call function, example with goo.gl URL shortner
+#' library(googleAuthR)
+#' options("googleAuthR.scopes.selected" = c("https://www.googleapis.com/auth/urlshortener"))
+#' 
+#' shorten_url <- function(url){
+#' 
+#'   body = list(
+#'     longUrl = url
+#'  )
+#'  
+#'  f <- gar_api_generator("https://www.googleapis.com/urlshortener/v1/url",
+#'                         "POST",
+#'                         data_parse_function = function(x) x$id)
+#'                         
+#'  f(the_body = body)
+#'  
+#'  }
+#' 
+#' 
+#' ## in server.R
+#' library(shiny)
+#' library(googleAuthR)
+#' source('global.R')
+#' 
+#' shinyServer(function(input, output, session)){
+#'   
+#'   ## Get auth code from return URL
+#'   access_token  <- reactiveAccessToken(session)
+#' 
+#'   ## Make a loginButton to display using loginOutput
+#'   ## revoke=TRUE means upon logout a user will need to reauthenticate
+#'   output$loginButton <- renderLogin(session, access_token(), revoke=TRUE)
+#'   
+#'   ## Needed if revoke=TRUE above
+#'   revokeEventObserver(access_token())
+#'
+#'   short_url_output <- eventReactive(input$submit, {
+#'   ## wrap existing function with_shiny
+#'   ## pass the reactive token in shiny_access_token
+#'   ## pass other named arguments
+#'     short_url <- with_shiny(f = shorten_url, 
+#'                            shiny_access_token = access_token(),
+#'                            url=input$url)
+#'                            
+#'    })
+#'    
+#'    output$short_url <- renderText({
+#'    
+#'      short_url_output()
+#'      
+#'    })
+#'  }
+#' 
+#' ## in ui.R
+#' library(shiny)
+#' library(googleAuthR)
+#' 
+#' shinyUI(
+#'   fluidPage(
+#'     loginOutput("loginButton"),
+#'     textInput("url", "Enter URL"),
+#'     actionButton("submit", "Shorten URL"),
+#'     textOutput("short_url")
+#'     ))
+#' } 
+revokeEventObserver <- function(access_token, input){
+  
+  shiny::observeEvent(input$revoke, {
+
+        ## GETS the revoke URL for this user's access_token
+        httr::GET(httr::modify_url("https://accounts.google.com/o/oauth2/revoke",
+                                   query = 
+                                   list(token = 
+                                        shiny::isolate(access_token)$credentials$access_token)))
+        message("Revoked access")
+  })
+
+}
+
 #' Turn a googleAuthR data fetch function into a Shiny compatible one
 #' 
 #' @param f A function generated by \code{googleAuth_fetch_generator}.
